import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { USER_MESSAGE_PUBLISHER } from '@modules/user/user.di-tokens';
import { UserDeletedDomainEvent } from '../../domain/events/user-deleted.domain-event';
import { UserDeletedIntegrationEvent } from '../events/user-deleted.integration-event';
import { USER_DELETED_ROUTING_KEY } from '@modules/user/user.constants';

@Injectable()
export class PublishMessageWhenUserIsDeletedDomainEventHandler {
  constructor(
    @Inject(USER_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(UserDeletedDomainEvent.name, { async: true, promisify: true })
  async handle(event: UserDeletedDomainEvent): Promise<any> {
    const userDeletedIntegrationEvent = new UserDeletedIntegrationEvent({
      id: event.aggregateId,
      metadata: event.metadata,
    });
    this.messagePublisher.publish(
      USER_DELETED_ROUTING_KEY,
      JSON.stringify(userDeletedIntegrationEvent),
    );
  }
}
