import {
  DatabaseErrorException,
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { DeleteUserRequestDto } from './dtos/delete-user.request.dto';
import { DeleteUserCommand } from '@modules/user/core/application/commands/delete-user/delete-user.command';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class DeleteUserGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod('UserService', 'Delete')
  async delete(data: DeleteUserRequestDto): Promise<void> {
    try {
      await this.commandBus.execute(new DeleteUserCommand(data));
    } catch (error: any) {
      if (error instanceof NotFoundException)
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: error.message,
        });
      if (error instanceof DatabaseErrorException)
        throw new RpcException({
          code: RpcExceptionCode.INTERNAL,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
