import { UserCreatedDomainEvent } from '@modules/user/core/domain/events/user-created.domain-event';
import { UserDeletedDomainEvent } from '@modules/user/core/domain/events/user-deleted.domain-event';
import { UserUpdatedDomainEvent } from '@modules/user/core/domain/events/user-updated.domain-event';
import { UserEntity } from '@modules/user/core/domain/user.entity';
import {
  CreateUserProps,
  UpdateUserProps,
} from '@modules/user/core/domain/user.types';

const createUserProps: CreateUserProps = {
  firstName: 'John',
  lastName: 'Doe',
  email: 'john.doe@email.com',
  phone: '+33611223344',
};

const createUserPropsWithId: CreateUserProps = {
  id: '66378860-cb75-40de-b6eb-061faf923fee',
  firstName: 'John',
  lastName: 'Doe',
  email: 'john.doe@email.com',
  phone: '+33611223344',
};

const updateUserProps: UpdateUserProps = {
  firstName: 'Jane',
  lastName: 'Dane',
  email: 'jane.dane@email.com',
};

describe('User entity create', () => {
  it('should create a new user entity', async () => {
    const userEntity: UserEntity = UserEntity.create(createUserProps);
    expect(userEntity.id.length).toBe(36);
    expect(userEntity.getProps().email).toBe('john.doe@email.com');
    expect(userEntity.domainEvents.length).toBe(1);
    expect(userEntity.domainEvents[0]).toBeInstanceOf(UserCreatedDomainEvent);
  });
  it('should create a new user entity with a given id', async () => {
    const userEntity: UserEntity = UserEntity.create(createUserPropsWithId);
    expect(userEntity.id).toBe('66378860-cb75-40de-b6eb-061faf923fee');
    expect(userEntity.getProps().email).toBe('john.doe@email.com');
    expect(userEntity.domainEvents.length).toBe(1);
    expect(userEntity.domainEvents[0]).toBeInstanceOf(UserCreatedDomainEvent);
  });
});

describe('User entity update', () => {
  it('should update a user entity', async () => {
    const userEntity: UserEntity = UserEntity.create(createUserProps);
    userEntity.update(updateUserProps);
    expect(userEntity.getProps().firstName).toBe('Jane');
    expect(userEntity.getProps().lastName).toBe('Dane');
    expect(userEntity.getProps().email).toBe('jane.dane@email.com');
    // 2 events because UserEntity.create sends a UserCreatedDomainEvent
    expect(userEntity.domainEvents.length).toBe(2);
    expect(userEntity.domainEvents[1]).toBeInstanceOf(UserUpdatedDomainEvent);
  });
});

describe('User entity delete', () => {
  it('should delete a user entity', async () => {
    const userEntity: UserEntity = UserEntity.create(createUserProps);
    userEntity.delete();
    // 2 events because UserEntity.create sends a UserCreatedDomainEvent
    expect(userEntity.domainEvents.length).toBe(2);
    expect(userEntity.domainEvents[1]).toBeInstanceOf(UserDeletedDomainEvent);
  });
});
